
#include "badge_init.h"
#include "badge_flash.h"
#include <msp430x14x.h>

void flash_spi_write(unsigned char *pFlashBuffer, unsigned char opcode, unsigned long address, unsigned int dataLen);
void flash_spi_read(unsigned char *pFlashBuffer, unsigned char opcode, unsigned long address, unsigned int dataLen, unsigned char dontCares);

unsigned char badge_flash_status(void)
{
  unsigned char tmp_rx;
  IFG1 &= ~URXIFG0;
  while ((IFG1 & UTXIFG0) == 0);          // USART0 TX buffer ready?
  P2OUT &= ~DF_CS;                        // Latch DATAFLASH
  TXBUF0 = 0xD7;
  while ((IFG1 & URXIFG0) == 0);
  tmp_rx = RXBUF0;
  while ((IFG1 & UTXIFG0) == 0);          // USART0 TX buffer ready?
  TXBUF0 = 0x00;
  while ((UTCTL0 & TXEPT) == 0);
  P2OUT |= DF_CS;
  while ((IFG1 & URXIFG0) == 0);
  return(RXBUF0);
}

void badge_flash_cont_array_read(unsigned char *pFlashBuffer, unsigned long address, unsigned int dataLen)
{
  flash_spi_read(pFlashBuffer,0xE8,address,dataLen,4);
}

void badge_flash_main_mem_read(unsigned char *pFlashBuffer, unsigned long address, unsigned int dataLen)
{
  flash_spi_read(pFlashBuffer,0xD2,address,dataLen,4);
}

void badge_flash_buffer_read(unsigned char *pFlashBuffer, unsigned long address, unsigned int dataLen, unsigned char buffer2Use)
{
  if (buffer2Use == BUFFER1){
    flash_spi_read(pFlashBuffer,0xD4,address,dataLen,1);
  } else {
    flash_spi_read(pFlashBuffer,0xD6,address,dataLen,1);
  } 
}

void badge_flash_buffer_write(unsigned char *pFlashBuffer, unsigned long address, unsigned int dataLen, unsigned char buffer2Use)
{
  if (buffer2Use == BUFFER1){
    flash_spi_write(pFlashBuffer,0x84,address,dataLen);
  } else {
    flash_spi_write(pFlashBuffer,0x87,address,dataLen);
  } 
}
void badge_flash_buffer2main_mem(unsigned char *pFlashBuffer, unsigned long address, unsigned char buffer2Use)
{
  if (buffer2Use == BUFFER1){
    flash_spi_write(pFlashBuffer,0x83,address,0);
  } else {
    flash_spi_write(pFlashBuffer,0x86,address,0);
  } 
}

void badge_flash_buffer2main_mem_noerase(unsigned char *pFlashBuffer, unsigned long address, unsigned char buffer2Use)
{
  if (buffer2Use == BUFFER1){
    flash_spi_write(pFlashBuffer,0x88,address,0);
  } else {
    flash_spi_write(pFlashBuffer,0x89,address,0);
  } 
}

void badge_flash_main_mem_write(unsigned char *pFlashBuffer, unsigned long address, unsigned int dataLen, unsigned char buffer2Use)
{   
  if (buffer2Use == BUFFER1){
    flash_spi_write(pFlashBuffer,0x82,address,dataLen);
  } else {
    flash_spi_write(pFlashBuffer,0x85,address,dataLen);
  } 
}

void badge_flash_page_erase(unsigned char *pFlashBuffer, unsigned long address)
{
  flash_spi_write(pFlashBuffer,0x81,address,0);
}

void badge_flash_block_erase(unsigned char *pFlashBuffer, unsigned long address)
{
  flash_spi_write(pFlashBuffer,0x50,address,0);
}

void badge_flash_main_mem2buffer(unsigned char *pFlashBuffer, unsigned long address, unsigned char buffer2Use)
{
  if (buffer2Use == BUFFER1){
    flash_spi_write(pFlashBuffer,0x53,address,0);
  } else {
    flash_spi_write(pFlashBuffer,0x55,address,0);
  } 
}
void badge_flash_auto_rewrite(unsigned char *pFlashBuffer, unsigned long address, unsigned char buffer2Use)
{
  if (buffer2Use == BUFFER1){
    flash_spi_write(pFlashBuffer,0x58,address,0);
  } else {
    flash_spi_write(pFlashBuffer,0x59,address,0);
  } 
}

unsigned char badge_flash_buffer2main_compare(unsigned char *pFlashBuffer, unsigned long address, unsigned char buffer2Use)
{
  if (buffer2Use == BUFFER1){
    flash_spi_write(pFlashBuffer,0x60,address,0);
  } else {
    flash_spi_write(pFlashBuffer,0x61,address,0);
  } 
  while (!(P1IN & DF_RDY));
  return (badge_flash_status() & 0x40) ;
}
void flash_spi_write(unsigned char *pFlashBuffer, unsigned char opcode, unsigned long address, unsigned int dataLen)
{
  unsigned int i;
  unsigned char addr1,addr2,addr3;

  while (!(P1IN & DF_RDY));

  IFG1 &= ~URXIFG0;
  while ((IFG1 & UTXIFG0) == 0);          // USART0 TX buffer ready?
  P2OUT &= ~DF_CS;                        // Latch DATAFLASH
  TXBUF0 = opcode;
  while ((IFG1 & UTXIFG0) == 0);          // USART0 TX buffer ready?
  TXBUF0 = (address & 0x003F0000) >> 16;
  while ((IFG1 & UTXIFG0) == 0);          // USART0 TX buffer ready?
  TXBUF0 = (address & 0x0000FF00) >> 8;
  while ((IFG1 & UTXIFG0) == 0);          // USART0 TX buffer ready?
  TXBUF0 = (address & 0x000000FF);
  for(i=0;i<dataLen;i++){
    while ((IFG1 & UTXIFG0) == 0);          // USART0 TX buffer ready?
    TXBUF0 = pFlashBuffer[i];
  }
  while ((UTCTL0 & TXEPT) == 0);
  P2OUT |= DF_CS;
}

void flash_spi_read(unsigned char *pFlashBuffer, unsigned char opcode, unsigned long address, unsigned int dataLen, unsigned char dontCares)
{
  unsigned int i;

  while (!(P1IN & DF_RDY));

  IFG1 &= ~URXIFG0;
  while ((IFG1 & UTXIFG0) == 0);          // USART0 TX buffer ready?
  P2OUT &= ~DF_CS;                        // Latch DATAFLASH
  TXBUF0 = opcode;
  while ((IFG1 & UTXIFG0) == 0);          // USART0 TX buffer ready?
  TXBUF0 = (address & 0x003F0000) >> 16;
  while ((IFG1 & UTXIFG0) == 0);          // USART0 TX buffer ready?
  TXBUF0 = (address & 0x0000FF00) >> 8;
  while ((IFG1 & UTXIFG0) == 0);          // USART0 TX buffer ready?
  TXBUF0 = (address & 0x000000FF);
  for(i=0;i<dontCares;i++) {
    while ((IFG1 & UTXIFG0) == 0);          // USART0 TX buffer ready?
    TXBUF0 = 0x00;
  }
  for(i=0;i<dataLen;i++){
    while ((IFG1 & UTXIFG0) == 0);          // USART0 TX buffer ready?
    TXBUF0 = 0x00;
    while ((UTCTL0 & TXEPT) == 0);
    while ((IFG1 & URXIFG0) == 0);
    pFlashBuffer[i] = RXBUF0;
  }
  while ((UTCTL0 & TXEPT) == 0);
  P2OUT |= DF_CS;
//  while ((IFG1 & URXIFG0) == 0);
//  pFlashBuffer[i] = RXBUF0;
}

