
#include "badge_init.h"
#include <msp430x14x.h>

void badge_init_osc(void) 
{
  unsigned int i;

  WDTCTL = WDTPW + WDTHOLD;             // Stop watchdog timer
  DCOCTL = 0xE0;                        // DCO AT MAX
 
  _BIC_SR(OSCOFF); 
  
  BCSCTL1 = XTS+RSEL0+RSEL1+RSEL2;      // ACLK = LFXT1 = HF XTAL
  BCSCTL1 |= XT2OFF;                    // SMCLK = DCO
    
  do {
    IFG1 &= ~OFIFG;                       // Clear OSCFault flag
    for (i = 0xFF; i > 0; i--);           // Time for flag to set
  } while ((IFG1 & OFIFG) != 0);          // OSCFault flag still set?                

  BCSCTL2 = SELM1+SELM0;               // MCLK = LFXT1 (safe)
}

void badge_init_io(void)
{
  P1DIR = 0x00;                         // Set P1 to all inputs
  P1DIR &= ~(DF_RDY);                   // Set DF_RDY to input
  P1SEL = 0x00;                         // Set P1 to all gpio
  P1OUT = DF_RDY;                       

  P2DIR = DF_CS + IRU_CS + LED_CS + DF_RST; // P2 output for CSs
  P2DIR &= ~(SW_UP + SW_DOWN);          // Switches to input
  P2SEL = 0x00;                         // P2 all gpio
  P2OUT = (DF_CS + IRU_CS + LED_CS + DF_RST + SW_UP + SW_DOWN);

  P3DIR = (SPI_MOSI + SPI_CLK + RF_TX);        // SPI MOSI AND CLK TO OUTPUT
  P3DIR &= ~(SPI_MISO + RF_RX);         // SPI MISO input UART overridden by SEL
  P3SEL = 0x00; // Enable 

  // ENABLE OUTPUTS ON PORT4
  P4DIR = (MOTOR + AUD_OUT1 + AUD_OUT2 + BLED1 + BLED2 + BLED3 + BLED4 );
  P4SEL = 0x00;
  P4OUT = 0x00;
  
  // P5 USED BY LCD INITIALIZED IN LCD LIBRARY IF INCLUDED
  P5DIR = 0x00;
  P5SEL = 0x00;
  P5OUT = 0x00;
  
  P6DIR = 0x00;
  P6DIR &= ~(AUD_FB + AUD_IN);          // AUDIO INPUTS
  P6SEL = (AUD_FB + AUD_IN);            // ENABLED ADC PERIPHERALS
  
}

void badge_init_spi(void)
{
  P3SEL |= (SPI_CLK + SPI_MOSI + SPI_MISO); // Enable 

  ME1 |= USPIE0;                        // Enable USART0 SPI mode
  UTCTL0 = SSEL1+SSEL0+STC+CKPH;        // SMCLK, 3-pin mode
  UCTL0 = CHAR+SYNC+MM+SWRST;           // 8-bit SPI Master **SWRST**
  UBR00 = 0xFF;                         // UCLK/2 
  UBR10 = 0x00;                         // 0
  UMCTL0 = 0x00;                        // no modulation
  UCTL0 &= ~SWRST;
}

void badge_init_uart(void)
{
  P3SEL |= (RF_RX + RF_TX); // Enable 
  
  UCTL1 = CHAR+SWRST;
  UTCTL1 = SSEL0;
  UBR01 = 0x8B;
  UBR11 = 0x00;
  UMCTL1 = 0x00;
  ME2 |= UTXE1 + URXE1;
  UCTL1 &= ~SWRST;
  IE2 |= UTXIE1 + URXIE1;
}

void badge_init_pwm(void)
{
   // ENABLE OUTPUTS ON PORT4
  P4SEL |= (AUD_OUT1 + AUD_OUT2 + BLED1 + BLED2 + BLED3 + BLED4);

  TBCTL = TBSSEL0 + TBCLR;              // ACLK, Clear Tar
  TBCCTL0 = OUTMOD_4 | CCIE;		// CCR0 toggle
  TBCCTL1 = OUTMOD_7 | CLLD_1;  	// CCR1 reset/set
  TBCCTL2 = OUTMOD_7 | CLLD_1;  	// CCR2 reset/set
  TBCCTL3 = OUTMOD_7;  			// CCR3 reset/set
  TBCCTL4 = OUTMOD_7;  			// CCR4 reset/set
  TBCCTL5 = OUTMOD_7;  			// CCR5 reset/set
  TBCCTL6 = OUTMOD_7;  			// CCR6 reset/set
  TBCCR0 = BIT_TIME-1;                  // CCR0 PWM duty cycle
  TBCCR1 = 0;                           // CCR1 PWM duty cycle
  TBCCR2 = BIT_TIME/2;                  // CCR2 PWM duty cycle
  TBCCR3 = 0;                           // CCR3 PWM duty cycle
  TBCCR4 = 0;                           // CCR4 PWM duty cycle
  TBCCR5 = 0;                           // CCR5 PWM duty cycle
  TBCCR6 = 0;                           // CCR6 PWM duty cycle
  TBCTL |= MC0;                         // Start Timer_B in up mode
}

