
#ifndef BADGE_INIT_H
#define BADGE_INIT_H

#define SYSCLK 8000000
#define DATA_BUFFER_LENGTH 64

#define DF_RDY    0x01              // DATAFLASH READY INDICATOR P1.0
#define BL1       0x04              // LCD1 BACKLIGHT ENABLE P1.2
#define BL2       0x08              // LCD2 BACKLIGHT ENABLE P1.3
#define RS        0x10              // LCD RS P1.4
#define RW        0x80              // LCD RW P1.7
#define DF_CS     0x01              // DATAFLASH CHIP SELECT P2.0
#define IRU_CS    0x02              // IR/UART CHIP SELECT P2.1
#define LED_CS    0x04              // LED DRIVER CHIP SELECT P2.2
#define DF_RST    0x08              // DATAFLASH RESET CONTROL P2.3
#define SW_DOWN   0x10              // SWITCH UP P2.4
#define SW_UP     0x20              // SWITCH UP P2.5
#define E1        0x40              // LCD1 ENABLE P2.6
#define E2        0x80              // LCD2 ENABLE P2.7
#define SPI_MOSI  0x02              // SPI MOSI P3.1
#define SPI_MISO  0x04              // SPI MISO P3.2
#define SPI_CLK   0x08              // SPI CLK P3.3
#define RF_TX     0x40              // UART TX TO RF P3.6/UTXD1
#define RF_RX     0x80              // UART RX TO RF P3.7/URXD1
#define MOTOR     0x01              // VIBRATOR MOTOR P4.0/TB0
#define AUD_OUT1  0x02              // PWM HALF AUDIO OUT P4.1/TB1 
#define AUD_OUT2  0x04              // PWM HALF AUDIO OUT P4.2/TB2 
#define BLED1     0x08              // BLUE LED 1 P4.3/TB3
#define BLED2     0x10              // BLUE LED 2 P4.4/TB4
#define BLED3     0x20              // BLUE LED 3 P4.5/TB5
#define BLED4     0x40              // BLUE LED 4 P4.6/TB6
#define AUD_FB    0x01              // AUDIO FEEDBACK  P6.0/A0
#define AUD_IN    0x02              // AUDIO IN P6.1/A1
#define LCD_DATA  P5OUT             // LCD DATA PORT

typedef unsigned char byte;
typedef unsigned short word;
typedef unsigned long ulong;

typedef byte bool;

#ifndef TRUE
    #define TRUE 1
#endif
#ifndef FALSE
    #define FALSE 0
#endif
#ifndef NULL
    #define NULL 0
#endif

#include  <msp430x14x.h>

#define CLOCK_RATE      8000                        //in KHz
#define SAMPLING_RATE	40                          //sampling in KHz
#define BIT_TIME	(CLOCK_RATE/SAMPLING_RATE)

#define V_MID_OFFSET	(1000*BIT_TIME/917/2)       //define offset to adjust the PWM DAC dc offset to mid of Vcc
                                                    //offset_count=0.5/(51K/(4.7K//300K+51K))*BIT_TIME

#define PWM1_MAX	63                          //max value of 6 bits
#define PWM2_MAX	((63*5*5*3)/(4*4*4))        //max value of 6 bits plus gain
                                                    //5=ADC input filter gain
                                                    //5=DAC output filter gain
                                                    //4,4,3/4 = internal multipliers

#define PWM1_OFFSET	(V_MID_OFFSET-(PWM1_MAX/2))
#define PWM2_OFFSET	(V_MID_OFFSET-(PWM2_MAX/2))


const unsigned char bitmask[8] = {0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01};

void badge_init_osc(void);
void badge_init_io(void);
void badge_init_spi(void);
void badge_init_uart(void);
void badge_init_pwm(void);

#endif
