#include "badge_leds.h"
#include <msp430x14x.h>

unsigned char align_byte(unsigned char in); 
unsigned char rol(unsigned char in,unsigned int count); 
unsigned char ror(unsigned char in,unsigned int count); 
void write_spi_led(unsigned char byte1, unsigned char byte2);

void badge_single_led (unsigned char led, unsigned char brightness)
{
  if (led == BLED1) {
    TBCCR3 = brightness;                           // CCR3 PWM duty cycle
  } else if (led == BLED2) {
    TBCCR4 = brightness;                           // CCR4 PWM duty cycle
  } else if (led == BLED3) {
    TBCCR5 = brightness;                           // CCR5 PWM duty cycle
  } else if (led == BLED4) {
    TBCCR6 = brightness;                           // CCR6 PWM duty cycle
  }
}

void badge_led_grid_init (void)
{
  write_spi_led(0x01,0x00);
  write_spi_led(0x02,0x0F);
  write_spi_led(0x03,0x05);
  write_spi_led(0x07,0x00);
  write_spi_led(0x04,0x01);
}

void badge_led_grid_intensity (unsigned char intensity)
{
  write_spi_led(0x02,intensity);
}

void badge_led_grid_shutdown (void) 
{
  write_spi_led(0x04,0x00);
}

void badge_led_grid_clear (void)
{
  write_spi_led(0x04,0x21);
}
void badge_led_grid_blink (unsigned char on_off, unsigned char speed)
{
  unsigned char tmp_val;
  if (on_off == FALSE) {
    write_spi_led(0x04,0x01);
  } else {
    if (speed == BLINK_FAST) {
      write_spi_led(0x04,0x0D);
    } else {
      write_spi_led(0x04,0x09);
    }
  }
}

void badge_led_grid_draw_screen (unsigned int row1, unsigned int row2, unsigned int row3, unsigned int row4, unsigned int row5)
{
  unsigned char temp=0;

  write_spi_led(0x60,align_byte(row1 & 0x00FF));
  write_spi_led(0x61,align_byte(row2 & 0x00FF));
  write_spi_led(0x62,align_byte(row3 & 0x00FF));
  write_spi_led(0x63,align_byte(row4 & 0x00FF));
  write_spi_led(0x64,align_byte(row5 & 0x00FF));
  
  if (row1 & 0x100) temp |= 0x04;
  if (row2 & 0x100) temp |= 0x08;
  if (row3 & 0x100) temp |= 0x10;
  if (row4 & 0x100) temp |= 0x20;
  if (row5 & 0x100) temp |= 0x40;
  
  write_spi_led(0x65,temp);
}
void badge_led_grid_draw_screen_2 (unsigned int row1, unsigned int row2, unsigned int row3, unsigned int row4, unsigned int row5)
{
  unsigned char temp=0;

  write_spi_led(0x40,align_byte(row1 & 0x00FF));
  write_spi_led(0x41,align_byte(row2 & 0x00FF));
  write_spi_led(0x42,align_byte(row3 & 0x00FF));
  write_spi_led(0x43,align_byte(row4 & 0x00FF));
  write_spi_led(0x44,align_byte(row5 & 0x00FF));
  
  if (row1 & 0x100) temp |= 0x04;
  if (row2 & 0x100) temp |= 0x08;
  if (row3 & 0x100) temp |= 0x10;
  if (row4 & 0x100) temp |= 0x20;
  if (row5 & 0x100) temp |= 0x40;
  
  write_spi_led(0x45,temp);
}


void badge_led_grid_draw_row (unsigned char row_num, unsigned int row_data)
{
  if (row_num < 6){
    write_spi_led(0x20+row_num,align_byte(row_data));
  }
}


void badge_led_grid_draw_row_2 (unsigned char row_num, unsigned int row_data)
{
  if (row_num < 6){
    write_spi_led(0x40+row_num,align_byte(row_data));
  }
}

unsigned char align_byte(unsigned char in) 
{
  unsigned char temp,bit;

  in = rol(in,1);
  temp = 0;
  for (bit=0;bit<8;bit++)
    {
      if(in & bitmask[bit]) {
        temp |= bitmask[7-bit];
      }
    }
  return temp;
}

unsigned char rol(unsigned char in,unsigned int count) 
{
  return (in << count | in >> (8-count));
}

unsigned char ror(unsigned char in,unsigned int count) 
{
  return (in >> count | in << (8-count));
}

void write_spi_led (unsigned char byte1, unsigned char byte2)
{
  while ((IFG1 & UTXIFG0) == 0);        // USART0 TX buffer ready?
  P2OUT &= ~LED_CS;                       // Latch LED
  TXBUF0 = byte1;
  while ((IFG1 & UTXIFG0) == 0);        // USART0 TX buffer ready?
  TXBUF0 = byte2;
  while ((UTCTL0 & TXEPT) == 0);
  P2OUT |= LED_CS;
}
