
#include "badge_init.h"
#include "badge_isr.h"
#include "badge_rf.h"
#include <string.h>

void init_rf_state_machine(void)
{
  badgeRFState.current_pid=0;
  badgeRFState.rf_uart_state=0;
  badgeRFState.rf_uart_bytes=0; 
}
void send_rf_packet(unsigned char flags,unsigned char destH, unsigned char destL, unsigned char dataLen, unsigned char *dataTp)
{
  unsigned int i;
  send_byte(0xFF);
  send_byte(++badgeRFState.current_pid);
  send_byte(0x08);
  send_byte(destH);
  send_byte(destL);
  send_byte(dataLen);
  send_byte(flags);
  for (i=0;i<dataLen;i++) {
    send_byte(dataTp[i]);
    }
}

void set_rf_settings(unsigned char addrH, unsigned char addrL, unsigned char ackTO, unsigned char ackAttempts)
{
  send_byte(0xFF);
  send_byte(++badgeRFState.current_pid);
  send_byte(0x03);
  send_byte(addrH);
  send_byte(addrL);
  send_byte(ackTO);
  send_byte(ackAttempts);

}

void get_rf_status(void)
{
  send_byte(0xFF);
  send_byte(++badgeRFState.current_pid);
  send_byte(0x04);
}

void get_rf_packet(void)
{
  send_byte(0xFF);
  send_byte(++badgeRFState.current_pid);
  send_byte(0x07);
}

void turn_rf_off(void)
{
  send_byte(0xFF);
  send_byte(++badgeRFState.current_pid);
  send_byte(0x02);
}

void turn_rf_on(void)
{
  send_byte(0xFF);
  send_byte(++badgeRFState.current_pid);
  send_byte(0x01);
}

void clear_rf_tx_buffer(void)
{
  send_byte(0xFF);
  send_byte(++badgeRFState.current_pid);
  send_byte(0x05);
}

void clear_rf_rx_buffer(void)
{
  send_byte(0xFF);
  send_byte(++badgeRFState.current_pid);
  send_byte(0x06);
}

unsigned char handle_rf_byte(unsigned char new_byte, unsigned char *newStatus, unsigned char *rfPacket)
{
  if (badgeRFState.rf_uart_state == 0) {
    if (new_byte == 0xFF) {
      // NEW HEADER BYTE
      badgeRFState.rf_uart_state = 1;
      return (NOTHING_NEW);
    }
  } else if (badgeRFState.rf_uart_state == 1) {  
    // PID RECEIVED
    badgeRFState.incoming_packet[0] = new_byte;
    badgeRFState.rf_uart_state = 2;
  } else if (badgeRFState.rf_uart_state == 2) {  
    // OPCODE RECEIVED
    badgeRFState.incoming_packet[1] = new_byte;
    badgeRFState.rf_uart_state = 3;
  } else if (badgeRFState.rf_uart_state == 3) {       
    badgeRFState.rf_uart_state = 0;
    if (badgeRFState.incoming_packet[1] == SEND_TX_OPCODE) {
      if (new_byte == 0xFE) {
        return (TX_FAILED);
      } else if (new_byte == 0xFD) {
        return (TX_BUFFER_FULL);
      } else {
        return (TX_OKAY);
      }
    } else if (badgeRFState.incoming_packet[1] == GET_SETTINGS_OPCODE) {
      if (new_byte == 0xFE) {
        return (RF_SETTINGS_FAILED);
      } else {
        return (RF_SETTINGS_OKAY);
      }
    } else if (badgeRFState.incoming_packet[1] == RF_ON_OPCODE) {
      if (new_byte == 0xFE) {
        return (RF_ON_FAILED);
      } else {
        return (RF_ON);
      }
    } else if (badgeRFState.incoming_packet[1] == RF_OFF_OPCODE) {
      if (new_byte == 0xFE) {
        return (RF_OFF_FAILED);
      } else {
        return (RF_OFF);
      }
    } else if (badgeRFState.incoming_packet[1] == RX_BUFFER_CLEAR_OPCODE) {
      if (new_byte == 0xFE) {
        return (RX_BUFFER_CLEAR_FAILED);
      } else {
        return (RX_BUFFER_CLEAR);
      }
    } else if (badgeRFState.incoming_packet[1] == TX_BUFFER_CLEAR_OPCODE) {
      if (new_byte == 0xFE) {
        return (TX_BUFFER_CLEAR_FAILED);
      } else {
        return (TX_BUFFER_CLEAR);
      }
    } else if (badgeRFState.incoming_packet[1] == GET_STATUS_OPCODE) {
      *newStatus = new_byte;
      return (NEW_STATUS);
    } else if (badgeRFState.incoming_packet[1] == GET_RX_PACKET_OPCODE) {
      badgeRFState.rf_uart_state = 4;
      badgeRFState.incoming_packet[0] = new_byte;
      badgeRFState.rf_uart_bytes = 1;
      return (NOTHING_NEW);
    } 
  } else if (badgeRFState.rf_uart_state == 4) {       
    badgeRFState.incoming_packet[badgeRFState.rf_uart_bytes++] = new_byte;
    if (badgeRFState.rf_uart_bytes == 4) {
      if (badgeRFState.incoming_packet[2]) {
        badgeRFState.rf_uart_state = 5;
      } else {
        badgeRFState.rf_uart_state = 0;
        memcpy(rfPacket,badgeRFState.incoming_packet,badgeRFState.rf_uart_bytes);
        return (NEW_RX_PACKET);
      }
    }
  } else if (badgeRFState.rf_uart_state == 5) {       
    badgeRFState.incoming_packet[badgeRFState.rf_uart_bytes++] = new_byte;
    if (badgeRFState.rf_uart_bytes == (badgeRFState.incoming_packet[2] + 4)) {
      badgeRFState.rf_uart_state = 0;
      memcpy(rfPacket,badgeRFState.incoming_packet,badgeRFState.rf_uart_bytes);
      return (NEW_RX_PACKET);
    }
  }  
  
  return (NOTHING_NEW);
}

